==> jupyter-0.11.0-40-snapshot.jar: ksql/data_api.clj <==
(ns ksql.data-api
  (:require [clojure.tools.logging :as log]


            [ksql.core :as ksql]
            [ksqldb.client :as client]
            [ksqldb.client.broker-api :as bapi ]
            [ksqldb.client.broker-sedes-config :as bsc]
            [ksql.gen.protocol :as p]

            [ksql.ksqldb-api :as dapi]
            [ksql.gen-api :as gapi]
            ))



(defn push-data-batch
  [strem-name rows]
  (client/push-data-batch (ksql/get-context) strem-name rows))


(defn as-entity-name [f-name]
  (let [w (clojure.string/split f-name #"/")]
    (if-let [t (last w)]
      (first (clojure.string/split t #"\."))
      f-name)))


(defn as-topic-data [model-coll]
  (doall
    (for [w model-coll]
      (let [[ds_file data_file key entity_name] w
            data_file (as-entity-name data_file)

            v {:ds-name     (as-entity-name ds_file)
               :entity_name (or data_file entity_name)
               :key         key}
            k (or entity_name data_file)]
        {:key k :value v}))))


(defn push-edn-event+ [context topic-name edn-data-coll]
  (let [c (bsc/edn-topic-config context topic-name)]
    (doseq [{:keys [key value]} edn-data-coll #_(as-topic-data)]
      (bapi/publish context c key value))))


(defn push-stardog [context model-coll]
  (log/info "Push stardog kafka context " context)
  (let [w (mapv as-topic-data model-coll)]
    (push-edn-event+ context "_pipeline_metadata" w)
    ))




#_(defmulti push-file (fn [_ file-path]

                      (let [w (last (clojure.string/split file-path #"\."))]
                        (clojure.string/lower-case w)
                        )

                      ))


#_(defmethod push-file :default
  [topic-name file-path]
  (let [w (last (clojure.string/split file-path #"\."))]
    (throw (Exception. "now support only json, xml and csv file " {:provided w
                                                                   :expected #{"json" "xml" "csv"}
                                                                   }))
    )

  )

#_(defmethod push-file "json"
  [topic-name json-file-path]
  (log/info " ---------start push file---------- " json-file-path)
  (let [context (ksql/get-context)
        topic-config (bsc/json-topic-config context topic-name)
        input-chan (bapi/publish-async context topic-config)
        total (atom 0)
        mapping [topic-name "_" (str "(from_json_file \"" json-file-path "\" )")
                 topic-name "_" "(value_format 'json' )" ";"
                 ]
        ]
    (p/invoke-dataflow "create-dataflow" context mapping)
    (try
      (with-open [rdr (clojure.java.io/reader json-file-path)]
        (doseq [line (line-seq rdr)]
          (async/>!! input-chan {:value line})
          (swap! total inc))
        (async/>!! input-chan "done!"))
      (catch Exception e
        (do
          (async/>!! input-chan "done!")
          (ex-data e))))
    (log/info "--------- End push file to kafka with " {:file-path json-file-path :total @total})))



#_(defmethod push-file "csv"
  [topic-name csv-file-path]
  (log/info " ---------start push file---------- " csv-file-path)
  (let [context (ksql/get-context)
        topic-config (bsc/json-topic-config context topic-name)
        input-chan (bapi/publish-async context topic-config)
        total (atom 0)
        separator (r/get-separator csv-file-path)
        header (r/read-csv-header (clojure.string/replace csv-file-path "'" ""))
        header (r/format-csv-header header)
        mapping [topic-name "_" (str "(from_csv_file \"" csv-file-path "\" )")
                 topic-name "_" "(value_format 'json' )" ";"
                 ]

        ]
    (p/invoke-dataflow "create-dataflow" context mapping)
    (try
      (with-open [rdr (csv/read-csv (clojure.java.io/reader csv-file-path) :separator separator)  #_(clojure.java.io/reader csv-file-path)]
        (doseq [line (rest rdr)]
          (let [w (zipmap header line)
                w-json (json/generate-string w)]
            (async/>!! input-chan {:value w-json})
            (swap! total inc)))
        (async/>!! input-chan "done!"))
      (catch Exception e
        (do
          (async/>!! input-chan "done!")
          (ex-data e))))
    (log/info "--------- End push file to kafka with " {:file-path csv-file-path :total @total})))



#_(defmethod push-file
  "xml"
  [topic-name xml-file-path]
  (let [context (ksql/get-context)
        topic-config (bsc/json-topic-config context topic-name)
        input-chan (bapi/publish-async context topic-config)
        total (atom 0)
        mapping [topic-name "_" (str "(from_xml_file \"" xml-file-path "\" )" ";")]]
    (p/invoke-dataflow "create-dataflow" context mapping)
    (try
      (let [w (r/convert-xml-file-to-edn xml-file-path)  #_(-> rdr
                                                               xml/parse
                                                               (xmltojson/parse)
                                                               #_(json/generate-string))
            w1 (json/generate-string w)
            ]
        ; (clojure.pprint/pprint w)
        (async/>!! input-chan {:value w1})
        (swap! total inc)
        (async/>!! input-chan "done!")
        )
      #_(with-open [rdr (clojure.java.io/file xml-file-path)]

          (async/>!! input-chan "done!"))
      (catch Exception e
        (do
          (async/>!! input-chan "done!")
          (ex-data e))))
    (log/info "--------- End push file to kafka with " {:file-path xml-file-path :total @total})))






#_(defn undeploy-connector-batch [connector-list]
    (doseq [c connector-list]
      (invoke-kafka "delete-connector" c)))

(comment

  (def client-context {:kafka-rest-url "http://localhost:8082"
                       :schema-url     "http://localhost:8081"
                       :connector-url  "http://localhost:8083"
                       :ksql-url       "http://localhost:8088"
                       :kafka-broker   "localhost:29092"
                       :join-window    "WITHIN 6 MINUTES"
                       :replicas       1
                       :partitions     1
                       })




  (push-stardog
    client-context

    [["tnf_db" "ref_mapping" "global_value"]]

    )


  (ksql/invoke-kafka "show-schemas")

  (ksql/invoke-kafka "describe-schema" "ref_mapping-value")


  (client/consume client-context)


  (clojure.string/split "test.3.json" #"\.")

  )

==> jupyter-0.11.0-43-snapshot.jar: ksql/data_api.clj <==
(ns ksql.data-api
  (:require [clojure.tools.logging :as log]
            [ksql.core :as ksql]
            [ksqldb.client :as client]
            [ksqldb.client.broker-api :as bapi ]
            [ksql.gen.reader.csv-data-reader :as r]
            [ksqldb.client.broker-sedes-config :as bsc]
            [ksql.gen.protocol :as p]
            [ksql.ksqldb-api :as kapi]
            [ksql.core :as core]
            [cheshire.core :as json]
            [ksql.gen-api :as gapi]))


(defn terminate-query [query-id]
  (client/terminate-push-query (client/get-client (core/get-context)) query-id))


(defn- as-query
  ([v total]
   (if (< 1 (count (clojure.string/split v #" ")))
     v
     (str "select * from " v " emit changes" " limit " total " ;")
     ))
  ([v]
   (if (< 1 (count (clojure.string/split v #" ")))
     v
     (str "select * from " v " emit changes;"))))


(defn query
  ([^String query-str] (query query-str println "latest" 3000))
  ([^String query-str callback] (query query-str callback "latest" 3000))
  ([^String query-str callback offset] (query query-str callback offset 3000))
  ([^String query-str callback offset duration]
   (client/stream-query-async-with-duration (core/get-context) (as-query query-str) callback offset duration)))


(defn query-by-total
  ([^String query-str total]
   (query-by-total query-str total "earliest"))
  ([^String query-str total offset]
   (client/stream-query (client/get-client (core/get-context)) (as-query query-str total) total offset)))


(comment
  (clojure.string/split "asdf asdf" #" "))



(defn print-query
  ([^String query-str] (print-query query-str "latest" 3000))
  ([^String query-str offset] (print-query query-str offset 3000))
  ([^String query-str offset duration]
   (client/stream-query-async-with-duration (core/get-context) (as-query query-str) println offset duration)))



(defn print-topic
  ([v] (print-topic v "latest" 3000))
  ([v offset] (print-topic v offset 3000))
  ([v offset duration]
   (client/print-topic-by-time (core/get-context) v offset duration println)))


(defn push-data-batch
  [strem-name rows]
  (client/push-data-batch (ksql/get-context) strem-name rows))


(defn as-entity-name [f-name]
  (let [w (clojure.string/split f-name #"/")]
    (if-let [t (last w)]
      (first (clojure.string/split t #"\."))
      f-name)))


(defn as-topic-data [model-coll]
  (doall
    (for [w model-coll]
      (let [[ds_file data_file key entity_name] w
            data_file (as-entity-name data_file)

            v {:ds-name     (as-entity-name ds_file)
               :entity_name (or data_file entity_name)
               :key         key}
            k (or entity_name data_file)]
        {:key k :value v}))))


(defn push-edn-event+ [context topic-name edn-data-coll]
  (let [c (bsc/edn-topic-config context topic-name)]
    (doseq [{:keys [key value]} edn-data-coll #_(as-topic-data)]
      (bapi/publish context c key value))))


(defn push-stardog [context model-coll]
  (log/info "Push stardog kafka context " context)
  (let [w (mapv as-topic-data model-coll)]
    (push-edn-event+ context "_pipeline_metadata" w)
    ))



(defn push-edn-event [topic-name edn-data-coll]
  (let [context (core/get-context)
        w (gapi/gen-napping topic-name edn-data-coll)]

    (p/invoke-dataflow-impl "create-dataflow" context w)
    (kapi/push-data-to-topic topic-name edn-data-coll)))


(defn push-json-event [topic-name json-str]
  (push-edn-event topic-name (json/parse-string json-str true)))



(defn push-file [topic-name file-path]
  (let [context (core/get-context)
        mapping (gapi/gen-mapping-by-file topic-name file-path)]
    ;  (clojure.pprint/pprint mapping)
    (p/invoke-dataflow-impl "create-dataflow" context mapping)
    (kapi/push-file-to-topic topic-name file-path)))


(defn push-meta-model [directory-name]
  (let [context (core/get-context)
        v ["_" "_" (str "(from_csv_folder \"" directory-name "\" )")
           "_" "_" "(value_format 'json' )" ";"
           "_" "_" (str "(from_json_folder \"" directory-name "\" )" ";")
           "_" "_" (str "(from_xml_folder \"" directory-name "\" )" ";")]]
    (p/invoke-dataflow-impl "create-dataflow" context v)))


(defn push-directory [directory-name]
  (let [csv-file-list (r/get-all-file-name directory-name "glob:*.{csv}")
        json-file-list (r/get-all-file-name directory-name "glob:*.{json}")
        xml-file-list (r/get-all-file-name directory-name "glob:*.{xml}")]
    ;  (import-meta-model context directory-name)
    (doseq [[file-name file-path] csv-file-list]
      (push-file file-name file-path))
    (doseq [[file-name file-path] json-file-list]
      (push-file file-name file-path))
    (doseq [[file-name file-path] xml-file-list]
      (push-file file-name file-path))))



#_(defmulti push-file (fn [_ file-path]

                      (let [w (last (clojure.string/split file-path #"\."))]
                        (clojure.string/lower-case w)
                        )

                      ))


#_(defmethod push-file :default
  [topic-name file-path]
  (let [w (last (clojure.string/split file-path #"\."))]
    (throw (Exception. "now support only json, xml and csv file " {:provided w
                                                                   :expected #{"json" "xml" "csv"}
                                                                   }))
    )

  )

#_(defmethod push-file "json"
  [topic-name json-file-path]
  (log/info " ---------start push file---------- " json-file-path)
  (let [context (ksql/get-context)
        topic-config (bsc/json-topic-config context topic-name)
        input-chan (bapi/publish-async context topic-config)
        total (atom 0)
        mapping [topic-name "_" (str "(from_json_file \"" json-file-path "\" )")
                 topic-name "_" "(value_format 'json' )" ";"
                 ]
        ]
    (p/invoke-dataflow "create-dataflow" context mapping)
    (try
      (with-open [rdr (clojure.java.io/reader json-file-path)]
        (doseq [line (line-seq rdr)]
          (async/>!! input-chan {:value line})
          (swap! total inc))
        (async/>!! input-chan "done!"))
      (catch Exception e
        (do
          (async/>!! input-chan "done!")
          (ex-data e))))
    (log/info "--------- End push file to kafka with " {:file-path json-file-path :total @total})))



#_(defmethod push-file "csv"
  [topic-name csv-file-path]
  (log/info " ---------start push file---------- " csv-file-path)
  (let [context (ksql/get-context)
        topic-config (bsc/json-topic-config context topic-name)
        input-chan (bapi/publish-async context topic-config)
        total (atom 0)
        separator (r/get-separator csv-file-path)
        header (r/read-csv-header (clojure.string/replace csv-file-path "'" ""))
        header (r/format-csv-header header)
        mapping [topic-name "_" (str "(from_csv_file \"" csv-file-path "\" )")
                 topic-name "_" "(value_format 'json' )" ";"
                 ]

        ]
    (p/invoke-dataflow "create-dataflow" context mapping)
    (try
      (with-open [rdr (csv/read-csv (clojure.java.io/reader csv-file-path) :separator separator)  #_(clojure.java.io/reader csv-file-path)]
        (doseq [line (rest rdr)]
          (let [w (zipmap header line)
                w-json (json/generate-string w)]
            (async/>!! input-chan {:value w-json})
            (swap! total inc)))
        (async/>!! input-chan "done!"))
      (catch Exception e
        (do
          (async/>!! input-chan "done!")
          (ex-data e))))
    (log/info "--------- End push file to kafka with " {:file-path csv-file-path :total @total})))



#_(defmethod push-file
  "xml"
  [topic-name xml-file-path]
  (let [context (ksql/get-context)
        topic-config (bsc/json-topic-config context topic-name)
        input-chan (bapi/publish-async context topic-config)
        total (atom 0)
        mapping [topic-name "_" (str "(from_xml_file \"" xml-file-path "\" )" ";")]]
    (p/invoke-dataflow "create-dataflow" context mapping)
    (try
      (let [w (r/convert-xml-file-to-edn xml-file-path)  #_(-> rdr
                                                               xml/parse
                                                               (xmltojson/parse)
                                                               #_(json/generate-string))
            w1 (json/generate-string w)
            ]
        ; (clojure.pprint/pprint w)
        (async/>!! input-chan {:value w1})
        (swap! total inc)
        (async/>!! input-chan "done!")
        )
      #_(with-open [rdr (clojure.java.io/file xml-file-path)]

          (async/>!! input-chan "done!"))
      (catch Exception e
        (do
          (async/>!! input-chan "done!")
          (ex-data e))))
    (log/info "--------- End push file to kafka with " {:file-path xml-file-path :total @total})))






#_(defn undeploy-connector-batch [connector-list]
    (doseq [c connector-list]
      (invoke-kafka "delete-connector" c)))

(comment

  (def client-context {:kafka-rest-url "http://localhost:8082"
                       :schema-url     "http://localhost:8081"
                       :connector-url  "http://localhost:8083"
                       :ksql-url       "http://localhost:8088"
                       :kafka-broker   "localhost:29092"
                       :join-window    "WITHIN 6 MINUTES"
                       :replicas       1
                       :partitions     1
                       })




  (push-stardog
    client-context

    [["tnf_db" "ref_mapping" "global_value"]]

    )


  (ksql/invoke-kafka "show-schemas")

  (ksql/invoke-kafka "describe-schema" "ref_mapping-value")


  (client/consume client-context)


  (clojure.string/split "test.3.json" #"\.")

  )
